/// <reference types="node" />
import { KeyVal, StringOrBuffer } from './types';
import { EventEmitter } from 'events';
export declare type MaybeLocation = {
    location?: string;
};
export declare type EasierLevelDOWNBatchOpts<K, V> = ReadonlyArray<{
    type: 'put';
    key: K;
    value: V;
} | {
    type: 'del';
    key: K;
}>;
export declare type EasierLevelDOWNIteratorOpts<K, V> = {
    gt?: K;
    gte?: K;
    lt?: K;
    lte?: K;
    reverse?: boolean;
    limit?: number;
};
export declare class EasierLevelDOWNEmitter<K, V> extends EventEmitter {
    constructor();
    emitPut(key: K, value: V): boolean;
    emitDel(key: K): boolean;
    emitBatch(array: EasierLevelDOWNBatchOpts<K, V>): boolean;
    onPut(cb: (key: K, value: V) => void): this;
    onDel(cb: (key: K) => void): this;
    onBatch(cb: (array: EasierLevelDOWNBatchOpts<K, V>) => void): this;
}
export interface EasierLevelDOWN<K, V = any, O extends MaybeLocation = any> {
    get(k: K): Promise<V>;
    put(k: K, v: V): Promise<void>;
    del(k: K): Promise<void>;
    open?(opts: O): Promise<void>;
    close?(): Promise<void>;
    iterator(opts: EasierLevelDOWNIteratorOpts<K, V>): AsyncIterableIterator<KeyVal<K, V>>;
    batch?(opts: EasierLevelDOWNBatchOpts<K, V>): Promise<void>;
    encodeKey?(key: K): StringOrBuffer;
    encodeValue?(value: V): StringOrBuffer;
    decodeKey?(key: StringOrBuffer): K;
    decodeValue?(value: StringOrBuffer): V;
    post?(v: V): Promise<K>;
    changes?(): EasierLevelDOWNEmitter<K, V>;
}
