"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const abstract_leveldown_1 = require("abstract-leveldown");
const level_down_iterator_1 = require("./level-down-iterator");
class EasierAbstractLevelDOWN extends abstract_leveldown_1.AbstractLevelDOWN {
    constructor(db, location) {
        super(location);
        this._db = db;
        this._location = location;
    }
    _open(options, callback) {
        if (this._db.open !== undefined) {
            let opts = {};
            if (options !== undefined)
                opts = options;
            if (this._location !== undefined)
                opts.location = this._location;
            this._db.open(opts).then(() => process.nextTick(callback)).catch((err) => process.nextTick(callback, err));
        }
        else {
            process.nextTick(callback);
        }
    }
    _close(callback) {
        if (this._db.close !== undefined) {
            this._db.close().then(() => process.nextTick(callback)).catch((err) => process.nextTick(callback, err));
        }
        else {
            process.nextTick(callback);
        }
    }
    _get(key, options, callback) {
        this._db.get(this._decodeKey(key)).then((value) => {
            if (options.asBuffer)
                process.nextTick(callback, null, Buffer.from(String(this._encodeValue(value))));
            else
                process.nextTick(callback, null, this._encodeValue(value));
        }).catch((err) => {
            process.nextTick(callback, err);
        });
    }
    _put(key, val, options, callback) {
        this._db.put(this._decodeKey(key), this._decodeValue(val)).then(() => {
            process.nextTick(callback);
        }).catch((err) => {
            process.nextTick(callback, err);
        });
    }
    _del(key, options, callback) {
        this._db.del(this._decodeKey(key)).then(() => {
            process.nextTick(callback);
        }).catch((err) => {
            process.nextTick(callback, err);
        });
    }
    _batch(array, options, callback) {
        if (this._db.batch !== undefined) {
            this._db.batch(array.map((op) => {
                if (op.type === 'put') {
                    return {
                        type: op.type,
                        key: this._decodeKey(op.key),
                        value: this._decodeValue(op.value)
                    };
                }
                else if (op.type === 'del') {
                    return {
                        type: op.type,
                        key: this._decodeKey(op.key)
                    };
                }
                else
                    throw new Error(`Unrecognized batch operation '${op.type}'`);
            })).then(() => process.nextTick(callback)).catch((err) => process.nextTick(callback, err));
        }
        else {
            (() => __awaiter(this, void 0, void 0, function* () {
                for (const op of array) {
                    if (op.type === 'put') {
                        yield new Promise((reject, resolve) => {
                            this._put(op.key, op.value, options, ((err, res) => {
                                if (err !== null) {
                                    reject(err);
                                }
                                else {
                                    resolve(res);
                                }
                            }));
                        });
                    }
                    else if (op.type === 'del') {
                        yield new Promise((reject, resolve) => {
                            this._del(op.key, options, ((err, res) => {
                                if (err !== null) {
                                    reject(err);
                                }
                                else {
                                    resolve(res);
                                }
                            }));
                        });
                    }
                    else {
                        throw new Error(`Unrecognized batch operation '${op.type}'`);
                    }
                }
            }))().then((results) => {
                process.nextTick(callback, null, results);
            }).catch((err) => {
                process.nextTick(callback, err, null);
            });
        }
    }
    _iterator(options) {
        return new level_down_iterator_1.EasierAbstractLevelDOWNIterator(this, options);
    }
    _encodeKey(key) {
        if (this._db.encodeKey !== undefined)
            return this._db.encodeKey(key);
        return key;
    }
    _decodeKey(key) {
        if (this._db.decodeKey !== undefined)
            return this._db.decodeKey(key);
        return key;
    }
    _encodeValue(value) {
        if (this._db.encodeValue !== undefined)
            return this._db.encodeValue(value);
        return value;
    }
    _decodeValue(value) {
        if (this._db.decodeValue !== undefined)
            return this._db.decodeValue(value);
        return value;
    }
}
exports.EasierAbstractLevelDOWN = EasierAbstractLevelDOWN;
//# sourceMappingURL=level-down.js.map