"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const abstract_leveldown_1 = require("abstract-leveldown");
const leveldown_opts_1 = require("./leveldown-opts");
class EasierAbstractLevelDOWNIterator extends abstract_leveldown_1.AbstractIterator {
    constructor(db, options) {
        super(options);
        this.db = db;
        this._options = leveldown_opts_1.normalize_options(options);
        this._test = leveldown_opts_1.test_from_options(this._options);
        this._done = 0;
        this._iterator = this.db._easier.iterator(Object.assign({}, this._options, { lt: this.db._decodeKey(this._options.lt), lte: this.db._decodeKey(this._options.lte), gt: this.db._decodeKey(this._options.gt), gte: this.db._decodeKey(this._options.gte) }));
    }
    _next(callback) {
        if (this._done >= this._options.limit)
            return process.nextTick(callback);
        this._iterator.next().then(({ done, value }) => {
            if (done)
                process.nextTick(callback);
            else {
                let key = this.db._encodeKey(value.key);
                if (this._test(key)) {
                    ++this._done;
                    if (this._options.keyAsBuffer && !Buffer.isBuffer(key))
                        key = Buffer.from(String(key));
                    else if (!this._options.keyAsBuffer)
                        key = String(key);
                    let val = this.db._encodeValue(value.value);
                    if (this._options.valueAsBuffer && !Buffer.isBuffer(val))
                        val = Buffer.from(String(val));
                    else if (!this._options.valueAsBuffer)
                        val = String(val);
                    process.nextTick(callback, null, key, val);
                }
                else {
                    process.nextTick(this._next.bind(this), callback);
                }
            }
        }).catch((err) => process.nextTick(callback, err));
    }
    _end(callback) {
        process.nextTick(callback);
    }
}
exports.EasierAbstractLevelDOWNIterator = EasierAbstractLevelDOWNIterator;
//# sourceMappingURL=leveldown-iterator.js.map