"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const abstract_leveldown_1 = require("abstract-leveldown");
const v4_1 = __importDefault(require("uuid/v4"));
const abstract_1 = require("./abstract");
const leveldown_iterator_1 = require("./leveldown-iterator");
class EasierAbstractLevelDOWN extends abstract_leveldown_1.AbstractLevelDOWN {
    constructor(easier, location) {
        super(location);
        this._easier = easier;
        this._location = location;
        if (this._easier.changes === undefined)
            this._easier.changes = undefined;
    }
    _open(options, callback) {
        if (this._easier.open !== undefined) {
            let opts = {};
            if (options !== undefined)
                opts = options;
            if (this._location !== undefined)
                opts.location = this._location;
            this._easier.open(opts).then(() => process.nextTick(callback)).catch((err) => process.nextTick(callback, err));
        }
        else {
            process.nextTick(callback);
        }
    }
    _close(callback) {
        if (this._easier.close !== undefined) {
            this._easier.close().then(() => process.nextTick(callback)).catch((err) => process.nextTick(callback, err));
        }
        else {
            process.nextTick(callback);
        }
    }
    _get(key, options, callback) {
        this._easier.get(this._decodeKey(key)).then((value) => {
            if (options.asBuffer)
                process.nextTick(callback, null, Buffer.from(String(this._encodeValue(value))));
            else
                process.nextTick(callback, null, String(this._encodeValue(value)));
        }).catch((err) => {
            process.nextTick(callback, err);
        });
    }
    _put(key, val, options, callback) {
        this._easier.put(this._decodeKey(key), this._decodeValue(val)).then(() => {
            process.nextTick(callback);
        }).catch((err) => {
            process.nextTick(callback, err);
        });
    }
    _del(key, options, callback) {
        this._easier.del(this._decodeKey(key)).then(() => {
            process.nextTick(callback);
        }).catch((err) => {
            process.nextTick(callback, err);
        });
    }
    _batch(array, options, callback) {
        if (this._easier.batch !== undefined) {
            this._easier.batch(array.map((op) => {
                if (op.type === 'put') {
                    return {
                        type: op.type,
                        key: this._decodeKey(op.key),
                        value: this._decodeValue(op.value)
                    };
                }
                else if (op.type === 'del') {
                    return {
                        type: op.type,
                        key: this._decodeKey(op.key)
                    };
                }
                else
                    throw new Error(`Unrecognized batch operation '${op.type}'`);
            })).then(() => process.nextTick(callback)).catch((err) => process.nextTick(callback, err));
        }
        else {
            (() => __awaiter(this, void 0, void 0, function* () {
                for (const op of array) {
                    if (op.type === 'put') {
                        yield new Promise((reject, resolve) => {
                            this._put(op.key, op.value, options, ((err, res) => {
                                if (err !== null) {
                                    reject(err);
                                }
                                else {
                                    resolve(res);
                                }
                            }));
                        });
                    }
                    else if (op.type === 'del') {
                        yield new Promise((reject, resolve) => {
                            this._del(op.key, options, ((err, res) => {
                                if (err !== null) {
                                    reject(err);
                                }
                                else {
                                    resolve(res);
                                }
                            }));
                        });
                    }
                    else {
                        throw new Error(`Unrecognized batch operation '${op.type}'`);
                    }
                }
            }))().then((results) => {
                process.nextTick(callback, null, results);
            }).catch((err) => {
                process.nextTick(callback, err, null);
            });
        }
    }
    _iterator(options) {
        return new leveldown_iterator_1.EasierAbstractLevelDOWNIterator(this, options);
    }
    // generate key
    post(val) {
        if (this._easier.post !== undefined) {
            return this._easier.post(this._decodeValue(val)).then((key) => this._encodeKey(key));
        }
        else {
            const key = String(v4_1.default());
            return this._easier.put(this._decodeKey(key), this._decodeValue(val)).then(() => key);
        }
    }
    // Passthrough changes encoding underlying events
    changes() {
        const newEmitter = new abstract_1.EasierLevelDOWNEmitter();
        if (this._easier.changes !== undefined) {
            this._easier.changes().onPut((key, value) => newEmitter.emitPut(this._encodeKey(key), this._encodeValue(value))).onDel((key) => newEmitter.emitDel(this._encodeKey(key))).onBatch((array) => newEmitter.emitBatch(array.map((op) => {
                if (op.type === 'put') {
                    return {
                        type: op.type,
                        key: this._encodeKey(op.key),
                        value: this._encodeValue(op.value)
                    };
                }
                else if (op.type === 'del') {
                    return {
                        type: op.type,
                        key: this._encodeKey(op.key)
                    };
                }
                else
                    throw new Error(`Unrecognized batch operation '${op.type}'`);
            })));
        }
        return newEmitter;
    }
    _encodeKey(key) {
        if (this._easier.encodeKey !== undefined)
            return this._easier.encodeKey(key);
        return key;
    }
    _decodeKey(key) {
        if (this._easier.decodeKey !== undefined)
            return this._easier.decodeKey(key);
        return key;
    }
    _encodeValue(value) {
        if (this._easier.encodeValue !== undefined)
            return this._easier.encodeValue(value);
        return value;
    }
    _decodeValue(value) {
        if (this._easier.decodeValue !== undefined)
            return this._easier.decodeValue(value);
        return value;
    }
}
exports.EasierAbstractLevelDOWN = EasierAbstractLevelDOWN;
//# sourceMappingURL=leveldown.js.map