"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const callback_promise_1 = require("./callback-promise");
const v4_1 = __importDefault(require("uuid/v4"));
/**
 * LevelDOWNEasier helps us treat an existing AbstractLevelDOWN compliant store as an EasierLevelDOWN one!
 *  It plays smart if the store was in-fact actually originally derived from an EasierLevelDOWN store.
 */
class LevelDOWNEasier {
    constructor(leveldown) {
        this._leveldown = leveldown;
        // Reuse existing implementations if present to mitigate multiple-encapsulation code expansion
        if (this._leveldown._easier !== undefined && this._leveldown._easier.get !== undefined)
            this.get = this._leveldown._easier.get.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.put !== undefined)
            this.put = this._leveldown._easier.put.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.del !== undefined)
            this.del = this._leveldown._easier.del.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.iterator !== undefined)
            this.iterator = this._leveldown._easier.iterator.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.open !== undefined)
            this.open = this._leveldown._easier.open.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.close !== undefined)
            this.close = this._leveldown._easier.close.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.batch !== undefined)
            this.batch = this._leveldown._easier.batch.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.post !== undefined)
            this.post = this._leveldown._easier.post.bind(this._leveldown._easier);
        else if (this._leveldown.post !== undefined)
            this.post = this._leveldown.post.bind(this._leveldown);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.changes !== undefined)
            this.changes = this._leveldown._easier.changes.bind(this._leveldown._easier);
        else if (this._leveldown.changes !== undefined)
            this.changes = this._leveldown.changes.bind(this._leveldown);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.encodeKey !== undefined)
            this.encodeKey = this._leveldown._easier.encodeKey.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.encodeValue !== undefined)
            this.encodeValue = this._leveldown._easier.encodeValue.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.decodeKey !== undefined)
            this.decodeKey = this._leveldown._easier.decodeKey.bind(this._leveldown._easier);
        if (this._leveldown._easier !== undefined && this._leveldown._easier.decodeValue !== undefined)
            this.decodeValue = this._leveldown._easier.decodeValue.bind(this._leveldown._easier);
    }
    get(k) {
        return callback_promise_1.ErrorValueCallbackToPromiseA1(this._leveldown.get.bind(this._leveldown), k);
    }
    put(k, v) {
        return callback_promise_1.ErrorCallbackToPromiseA2(this._leveldown.put.bind(this._leveldown), k, v);
    }
    post(val) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = v4_1.default();
            yield this.put(key, val);
            return key;
        });
    }
    del(k) {
        return callback_promise_1.ErrorCallbackToPromiseA1(this._leveldown.del.bind(this._leveldown), k);
    }
    open(opts) {
        return callback_promise_1.ErrorCallbackToPromiseA1(this._leveldown.open.bind(this._leveldown), opts);
    }
    close() {
        return callback_promise_1.ErrorCallbackToPromise(this._leveldown.close.bind(this._leveldown));
    }
    iterator(opts) {
        return __asyncGenerator(this, arguments, function* iterator_1() {
            const it = this._leveldown.iterator(opts);
            while (true) {
                const kv = yield __await(callback_promise_1.ErrorKeyValueCallbackToPromise(it.next.bind(it)));
                if (kv.key === undefined && kv.value === undefined)
                    break;
                yield yield __await(kv);
            }
        });
    }
    batch(array) {
        return callback_promise_1.ErrorCallbackToPromiseA1(this._leveldown.batch.bind(this._leveldown), array);
    }
}
exports.LevelDOWNEasier = LevelDOWNEasier;
//# sourceMappingURL=reverse.js.map